

local moduleName = "HealLines"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Heal Lines"
} end)



----------------------------------
--      Local Declaration      --
----------------------------------



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName)

plugin.revision = tonumber(("$Revision: 40597 $"):match("%d+"))

plugin.options = {
	name = L[moduleName],
	args = {
		disabled = {
			type = "toggle",
			name = "Disable",
			desc = "Disable Heal Lines.",
            get  = "GetSetDisabled",
            set  = "GetSetDisabled"
		}
	}
}

plugin.defaultDB = {
	disabled = true
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	self:ScheduleRepeatingEvent("BunchOfBars_HealLines", self.UpdateAll, 0.1, self)
end


function plugin:OnInactive(frame, line)
	if line then
		line:Hide()
	end
end


function plugin:OnUpdate(frame, line)
	local class = select(2, UnitClass(frame.unit))

	if not UnitIsUnit(frame.unit, "player") and (class == "PALADIN" or class == "PRIEST" or class == "DRUID" or class == "SHAMAN") then
		if not line then
			line = frame:CreateTexture(nil, "OVERLAY")
			line:SetTexture("Interface\\Addons\\BunchOfBars\\Textures\\Line")
			frame.parts[self.name] = line
		end

		for u,f in pairs(self.core.frames) do
			if UnitIsUnit(u, frame.unit.."target") then
				self:Line(line, frame, f.parts["HealthBar"], f, RAID_CLASS_COLORS[class])
				return
			end
		end

		line:Hide()
	elseif line then
		line:Hide()
		frame.parts[self.name] = nil
	end
end


function plugin:Line(line, startf, endf, appel, color)
	local ses = startf:GetScale()
	local ees = appel:GetScale()
	
	local sx = startf:GetLeft() * ses
	local sy = (startf:GetBottom() + startf:GetTop()) / 2 * ses
	local ex = endf:GetLeft() * ees
	local ey = (endf:GetBottom() + endf:GetTop()) / 2 * ees

	local dx,dy = ex - sx, ey - sy
	local cx,cy = (sx + ex) / 2, (sy + ey) / 2
	if (dx < 0) then
		dx,dy = -dx,-dy
	end
	local length = sqrt(dx^2 + dy^2)

	if length <= 0 then
		self.core:Print("|rffff0000line of length 0!|r")
		return
	end

	local s,c = -dy / length, dx / length
	local sc = s * c
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
	if (dy >= 0) then
		Bwid = ((length * c) - (32 * s)) * (32/60)
		Bhgt = ((32 * c) - (length * s)) * (32/60)
		BLx, BLy, BRy = (32 / length) * sc, s^2, (length / 32) * sc
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx
		TRy = BRx
	else
		Bwid = ((length * c) + (32 * s)) * (32/60)
		Bhgt = ((32 * c) + (length * s)) * (32/60)
		BLx, BLy, BRx = s * s, -(length / 32) * sc, 1 + (32 / length) * sc
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy
		TRx = TLy
	end
	local r,g,b
	
	line:SetVertexColor(color.r, color.g, color.b, 0.5)
	line:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy)
	line:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", cx + Bwid, cy + Bhgt)
	line:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", cx - Bwid, cy - Bhgt)
	line:Show()
end


function plugin:GetSetDisabled(v)
	if type(v) == "nil" then return self.db.profile.disabled end

	self.db.profile.disabled = v

	self.core:ToggleModuleActive(self, not self.db.profile.disabled)
end
